<?php

namespace SM\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SM\Service\CdaService;
use SM\Service\SmNextDocumentoCdaService;
use SM\Traits\TResponse;
use Zeedhi\Framework\DataSource\DataSet;

class CdaController
{
    use TResponse;
    
    protected $CdaService;
    protected $DocumentoCdaService;
    
    public function __construct(CdaService $CdaService, SmNextDocumentoCdaService $DocumentoCdaService)
    {
        $this->CdaService = $CdaService;
        $this->DocumentoCdaService = $DocumentoCdaService;
    }
    
    public function getCda(Request $request)
    {
        try {
            $data = $this->CdaService->getCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getFuncionarios(Request $request)
    {
        try {
            $data = $this->CdaService->getFuncionarios($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPermissaoOperadorCda(Request $request)
    {
        try {
            $data = $this->CdaService->getPermissaoOperadorCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOperadorLogado()
    {
        try {
            $data = $this->CdaService->getOperadorLogado();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPeriodoApuraPonto(Request $request)
    {
        try {
            $data = $this->CdaService->getPeriodoApuraPonto($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getClienteCda(Request $request)
    {
        try {
            $data = $this->CdaService->getClienteCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getClienteRelacionamento()
    {
        try {
            $data = $this->CdaService->getClienteRelacionamento();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPropostaCda(Request $request)
    {
        try {
            $data = $this->CdaService->getPropostaCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getProjetoOperadorCliente(Request $request)
    {
        try {
            $data = $this->CdaService->getProjetoOperadorCliente($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTipoAtividadeCda(Request $request)
    {
        try {
            $data = $this->CdaService->getTipoAtividadeCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getProspectCda(Request $request)
    {
        try {
            $data = $this->CdaService->getProspectCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getInterlocutorProspectCda(Request $request)
    {
        try {
            $data = $this->CdaService->getInterlocutorProspectCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function getInterlocutorClienteCda(Request $request)
    {
        try {
            $data = $this->CdaService->getInterlocutorClienteCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLicencaUsoCda(Request $request)
    {
        try {
            $data = $this->CdaService->getLicencaUsoCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getContatosParceiroCda(Request $request)
    {
        try {
            $data = $this->CdaService->getContatosParceiroCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getMotivoEvoContCda(Request $request)
    {
        try {
            $data = $this->CdaService->getMotivoEvoContCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function getMotivoContatoCda(Request $request)
    {
        try {
            $data = $this->CdaService->getMotivoContatoCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSistemaCda(Request $request)
    {
        try {
            $data = $this->CdaService->getSistemaCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteParticipantes(Request $request)
    {
        try {
            $data = $this->CdaService->deleteParticipantes($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getParticipantesCda(Request $request)
    {
        try {
            $data = $this->CdaService->getParticipantesCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function inserirAtividadeCda(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->inserirAtividadeCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getHorasNaoFaturadas(Request $request)
    {
        try {
            $data = $this->CdaService->getHorasNaoFaturadas($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function existenciaCda(Request $request)
    {
        try {
            $data = $this->CdaService->existenciaCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getDescricaoCda(Request $request)
    {
        try {
            $data = $this->CdaService->getDescricaoCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function saveDescricaoAtividadeCda(Request $request)
    {
        try {
            $data = $this->CdaService->saveDescricaoAtividadeCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function confirmaOuCorrigeCda(Request $request)
    {
        try {
            $data = $this->CdaService->confirmaOuCorrigeCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deletarCda(Request $request)
    {
        try {
            $data = $this->CdaService->deletarCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function gerarDocumentoCda(Request $request, Response $response)
    {
        try {
            $data = $this->DocumentoCdaService->mergeDocCda($request->getParameters());
            $this->sendResponse(['link' => $data], 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function inserirRepresentante(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->inserirRepresentante($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 

    public function assinarDocumentoManualmente(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->assinarDocumentoManualmente($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function enviarDocumentoParaAssinatura(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->enviarDocumentoParaAssinatura($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getDadosAssinaturaCDA(Request $request)
    {
        try {
            $data = $this->CdaService->getDadosAssinaturaCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function getDadosAssinaturaCDAAssManual(Request $request)
    {
        try {
            $data = $this->CdaService->getDadosAssinaturaCDAAssManual($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function getDocsRelacionadosCDA(Request $request)
    {
        try {
            $data = $this->CdaService->getDocsRelacionadosCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getDadosSignatarioCDA(Request $request)
    {
        try {
            $data = $this->CdaService->getDadosSignatarioCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function salvarDadosSignatarioCDA(Request $request)
    {
        try {
            $data = $this->CdaService->salvarDadosSignatarioCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function criarLancamentosRecorrentes(Request $request)
    {
        try {
            $data = $this->CdaService->criarLancamentosRecorrentes($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * realiza o download do documento do ambiente Certsign 
    **/
    public function downloadDocEletrDigitalCDA(Request $request) {
        try{
           
            $params = $request->getParameters();
            $nrAssinaCDA = $params["NRSERVPRESTDOCASSINA"];

            $this->CdaService->downloadDocEletrDigitalCDA($nrAssinaCDA);

        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function enviarLembreteAssinatura(Request $request)
    {
        try {
            $data = $this->CdaService->enviarLembreteAssinatura($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function enviarLembreteAssinaturaAgrupado(Request $request)
    {
        try {
            $data = $this->CdaService->enviarLembreteAssinaturaAgrupado($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function enviarLembreteAutomatico(Request $request)
    {
        try {
            $data = $this->CdaService->enviarLembreteAutomatico($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function verificarAssinaturaCda(Request $request)
    {
        try {
            $data = $this->CdaService->verificarAssinaturaCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Métodos relacionados a nova estrutura da CDA utilizando as tabelas do SM

    public function getCdaAprop(Request $request)
    {
        try {
            $data = $this->CdaService->getCdaAprop($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getFuncionariosNrParceiro()
    {
        try {
            $data = $this->CdaService->getFuncionariosNrParceiro();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTipoLancamentoCda()
    {
        try {
            $data = $this->CdaService->getTipoLancamentoCda();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTipoLancamentoCdaLancRecorrente()
    {
        try {
            $data = $this->CdaService->getTipoLancamentoCdaLancRecorrente();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getParceiroClienteCda(Request $request)
    {
        try {
            $data = $this->CdaService->getParceiroClienteCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getParceiroProspectCda(Request $request)
    {
        try {
            $data = $this->CdaService->getParceiroProspectCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getParceiroTpRelacionamentoCda(Request $request)
    {
        try {
            $data = $this->CdaService->getParceiroTpRelacionamentoCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCrmInterlocutorCda(Request $request)
    {
        try {
            $data = $this->CdaService->getCrmInterlocutorCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function inserirInterlocutorCda(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->inserirInterlocutorCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getOrcamentoProspostaCda(Request $request)
    {
        try {
            $data = $this->CdaService->getOrcamentoProspostaCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getItemContratoCda(Request $request)
    {
        try {
            $data = $this->CdaService->getItemContratoCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getParticipantesApropCda(Request $request)
    {
        try {
            $data = $this->CdaService->getParticipantesApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteParticipantesAprop(Request $request)
    {
        try {
            $data = $this->CdaService->deleteParticipantesAprop($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getHorasNaoFaturadasAprop(Request $request)
    {
        try {
            $data = $this->CdaService->getHorasNaoFaturadasAprop($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function inserirAtividadeApropCda(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->inserirAtividadeApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deletarApropCda(Request $request)
    {
        try {
            $data = $this->CdaService->deletarApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function confirmaOuCorrigeApropCda(Request $request)
    {
        try {
            $data = $this->CdaService->confirmaOuCorrigeApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function verificarAssinaturaApropCda(Request $request)
    {
        try {
            $data = $this->CdaService->verificarAssinaturaApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function gerarDocumentoApropCda(Request $request, Response $response)
    {
        try {
            $data = $this->DocumentoCdaService->mergeDocApropCda($request->getParameters());
            $this->sendResponse(['link' => $data], 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvarDadosSignatarioApropCda(Request $request)
    {
        try {
            $data = $this->CdaService->salvarDadosSignatarioApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function enviarDocumentoParaAssinaturaApropCda(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->enviarDocumentoParaAssinaturaApropCda($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getDadosAssinaturaApropCDA(Request $request)
    {
        try {
            $data = $this->CdaService->getDadosAssinaturaApropCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function getDocsRelacionadosApropCDA(Request $request)
    {
        try {
            $data = $this->CdaService->getDocsRelacionadosApropCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function enviarLembreteAssinaturaApropCDA(Request $request)
    {
        try {
            $data = $this->CdaService->enviarLembreteAssinaturaApropCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getDadosAssinaturaAssManualApropCDA(Request $request)
    {
        try {
            $data = $this->CdaService->getDadosAssinaturaAssManualApropCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function assinarDocumentoManualmenteApropCDA(Request $request, Response $response)
    {
        try {
            $data = $this->CdaService->assinarDocumentoManualmenteApropCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function criarLancamentosRecorrentesApropCDA(Request $request)
    {
        try {
            $data = $this->CdaService->criarLancamentosRecorrentesApropCDA($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
