<?php
namespace SM\Controller;

use SM\Controller\SMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use SM\Service\Sm00029000LocalAtendimentoService;
use SM\Traits\TResponse;

class Sm00029000LocalAtendimentoController extends SMCrud
{
    use TResponse;

    protected $localAtendimentoService;

    public function __construct($crudService, $environment, Sm00029000LocalAtendimentoService $localAtendimentoService)
    {
        parent::__construct($crudService, $environment);
        $this->localAtendimentoService=$localAtendimentoService;
    }

    public function getTipoLocalAtendimento()
    {        
        try {
            $data = $this->localAtendimentoService->getTipoLocalAtendimento();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTipLocAtendEstruturam()
    {        
        try {
            $data = $this->localAtendimentoService->getTipLocAtendEstruturam();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findLocalAtendimentoCentroCusto()
    {
        try {
            $data = $this->localAtendimentoService->findLocalAtendimentoCentroCusto();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findPais()
    {
        try {
            $data = $this->localAtendimentoService->findPais();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findEstado(Request $request)
    {
        try {
            $data = $this->localAtendimentoService->findEstado($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findMunicipio(Request $request)
    {
        try {
            $data = $this->localAtendimentoService->findMunicipio($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteLocalAtendimento(Request $request)
    {
        try {
            $data = $this->localAtendimentoService->deleteLocalAtendimento($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getParcNegocioCliente()
    {        
        try {
            $data = $this->localAtendimentoService->getParcNegocioCliente();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLocalAtendimentoFilial()
    {        
        try {
            $data = $this->localAtendimentoService->getLocalAtendimentoFilial();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLocalAtendimentoEmpresa()
    {        
        try {
            $data = $this->localAtendimentoService->getLocalAtendimentoEmpresa();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLocalAtendimentoInscrEstad()
    {        
        try {
            $data = $this->localAtendimentoService->getLocalAtendimentoInscrEstad();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLocalAtendimentoInscrMunicip()
    {        
        try {
            $data = $this->localAtendimentoService->getLocalAtendimentoInscrMunicip();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    
    private function getFiltro($request)
    {
        if (isset($request->getParameters()["find"])) {
            $filtro = $request->getParameters()["find"];
        } else {
            $filtro = '';
        }
        return $filtro;
    }

    // public function getBairro(Request $request)
    // {        
    //     try {
    //         $filtro = $this->getFiltro($request);
    //         $parceiro = $request->getParameters();
    //         $data = $this->localAtendimentoService->getBairro($filtro, $parceiro);
    //         $this->sendResponse($data, 200);
    //     } catch (\Exception $ex) {
    //         $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    //     }
    // }

    public function getBairro(Request $request)
    {        
        try {
            $filtro = $this->getFiltro($request);
            $parceiro = $request->getParameters();
            $data = $this->localAtendimentoService->getBairro($filtro, $request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findCodeDistrict(Request $request)
    {
        try {
            $data = $this->localAtendimentoService->findCodeDistrict($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findBairro(Request $request)
    {
        try {
            $data = $this->localAtendimentoService->findBairro($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
