<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\Query\Expr;
use Teknisa\Model\Traits\TQuery;

class RelatorioCdaFactory
{
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getRelatorioCda()
    {
        $sql = "SELECT REL.NMOPERADORCDAREL, REL.CDOPERADORCDAREL,  REL.DTSERVPRESTCDAREL, REL.IDSERVPRESTCDAREL, REL.NMFANTRAZCDAREL,
                    REL.IDSITSERVPRESTCDAREL, REL.CLIENTECDAREL, REL.CDCLIENTE, REL.IDCLIEATIV, REL.CDINTECLIE,
                    REL.NMINTECLIECDAREL, REL.LOCALSERVCDAREL, REL.NRPROPOSFAS, REL.NRVERPROPFAS, REL.DSREFEPROPOSFASCDAREL,
                    REL.CDPROJETO, REL.CDTIPOSERV, REL.NMPROJETOCDAREL, REL.NMTIPOSERVCDAREL, REL.DTENTRADACDAREL,
                    REL.HRAUSENCIAALMOCOCDAREL, REL.DTSAIDACDAREL, REL.QTHRTOTALCDAREL,  REL.QTHRDESLOCAMENTOFATCDAREL,  REL.QTHRATIVIDADEFATCDAREL,
                    REL.DSJUSTHRNAOFATURADACDAREL, REL.SITUACAOASSCDAREL, REL.DATAASSCDAREL, REL.IDFALTACDAREL, 
                    REL.NRPARCNEGOCIO, REL.NRPERFACESSPARC, REL.NRPERFILACESSO, REL.NMPERFILACESSO
                FROM
                    (SELECT FUNCIONARIO.NMFUNC AS NMOPERADORCDAREL,
                    FUNCIONARIO.CDFUNC AS CDOPERADORCDAREL,
                    TO_CHAR(SERVPREST.DTSERVPREST, 'DD/MM/YYYY') AS DTSERVPRESTCDAREL,
                    SERVPREST.IDSERVPREST AS IDSERVPRESTCDAREL,
                    CLIENTE.NMFANTCLIE AS NMFANTRAZCDAREL,
                    NVL(SERVPREST.IDSITSERVPREST, 'P') AS IDSITSERVPRESTCDAREL,
                    CLIENTE.NMFANTCLIE AS CLIENTECDAREL,
                    CLIENTE.CDCLIENTE,
                    CLIENTE.IDCLIEATIV,
                    SERVPREST.CDINTECLIE,
                    SERVPREST.NMINTCLIENTE AS NMINTECLIECDAREL,
                    LICENCAUSO.NMFANTLUSO AS LOCALSERVCDAREL,
                    SERVPREST.NRPROPOSFAS,
                    SERVPREST.NRVERPROPFAS,
                    PROPOSTAFAS.NRPROPOSFAS || ' - ' || PROPOSTAFAS.DSREFEPROPOSFAS AS DSREFEPROPOSFASCDAREL,
                    PROJETO.CDPROJETO,
                    TIPOSERV.CDTIPOSERV,
                    PROJETO.NMPROJETO AS NMPROJETOCDAREL,
                    TIPOSERV.NMTIPOSERV AS NMTIPOSERVCDAREL,
                    TO_CHAR(SERVPREST.DTENTRADA, 'HH24:MI') AS DTENTRADACDAREL,
                    DECODE(SERVPREST.HRAUSENCIAALMOCO, NULL, '00:00', SUBSTR(SERVPREST.HRAUSENCIAALMOCO,1,2) || ':' || SUBSTR(SERVPREST.HRAUSENCIAALMOCO,-2,2)) AS HRAUSENCIAALMOCOCDAREL,
                    TO_CHAR(SERVPREST.DTSAIDA, 'HH24:MI') AS DTSAIDACDAREL,
                    DECODE(SERVPREST.QTHRTOTAL, NULL, '00:00', SUBSTR(SERVPREST.QTHRTOTAL,1,2) || ':' || SUBSTR(SERVPREST.QTHRTOTAL,-2,2)) AS QTHRTOTALCDAREL,
                    DECODE(SERVPREST.QTHRDESLOCAMENTOFAT, NULL, '00:00', SUBSTR(SERVPREST.QTHRDESLOCAMENTOFAT,1,2) || ':' || SUBSTR(SERVPREST.QTHRDESLOCAMENTOFAT,-2,2)) AS QTHRDESLOCAMENTOFATCDAREL,
                    DECODE(SERVPREST.QTHRATIVIDADEFAT, NULL, '00:00', SUBSTR(SERVPREST.QTHRATIVIDADEFAT,1,2) || ':' || SUBSTR(SERVPREST.QTHRATIVIDADEFAT,-2,2)) AS QTHRATIVIDADEFATCDAREL,
                    HORADEC_HORACHAR(HORACHAR_HORADEC(QTHRTOTAL) - HORACHAR_HORADEC(QTHRATIVIDADEFAT)) AS DSJUSTHRNAOFATURADACDAREL,
                    SERVPRESTDOCASSINA.IDSITASSINATURA AS SITUACAOASSCDAREL,
                    SERVPRESTDOCASSINA.DTASSINATURA AS DATAASSCDAREL,
                    CASE WHEN SERVPREST.IDSERVPREST = 'FALTA_NAO_JUSTIFICADA' THEN 'S' ELSE NULL END AS IDFALTACDAREL,
                    OPERADOR.NRPARCNEGOCIO,
                    SERVPREST.CDEMPRESA, 
                    SERVPREST.NRSERVPREST,
                    LISTAGG(PERFACESSPARC.NRPERFACESSPARC, ',') WITHIN GROUP(ORDER BY PERFACESSPARC.NRPERFACESSPARC) AS NRPERFACESSPARC,
                    LISTAGG(PERFILACESSO.NRPERFILACESSO, ',') WITHIN GROUP(ORDER BY PERFILACESSO.NRPERFILACESSO) AS NRPERFILACESSO,
                    LISTAGG(PERFILACESSO.NMPERFILACESSO, ',') WITHIN GROUP(ORDER BY PERFILACESSO.NMPERFILACESSO) AS NMPERFILACESSO
                FROM SERVPREST
                LEFT JOIN CLIENTE ON SERVPREST.CDCLIENTE = CLIENTE.CDCLIENTE
                LEFT JOIN FUNCIONARIO ON FUNCIONARIO.CDFUNC = SERVPREST.CDFUNC
                LEFT JOIN OPERADOR ON OPERADOR.CDFUNC = FUNCIONARIO.CDFUNC
                LEFT JOIN PARCNEGOCIO ON PARCNEGOCIO.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
                LEFT JOIN TIPOSERV ON TIPOSERV.CDTIPOSERV = SERVPREST.CDTIPOSERV
                LEFT JOIN LICENCAUSO ON LICENCAUSO.CDCLIENTE = SERVPREST.CDCLIENTE AND LICENCAUSO.NRLICENCAUSO = SERVPREST.NRLICENCAUSO
                LEFT JOIN PROPOSTAFAS ON PROPOSTAFAS.NRPROPOSFAS = SERVPREST.NRPROPOSFAS
                LEFT JOIN PROJETO ON SERVPREST.CDPROJETO = PROJETO.CDPROJETO
                LEFT JOIN DESCSERVPRES ON DESCSERVPRES.CDFUNC = SERVPREST.CDFUNC AND DESCSERVPRES.CDCLIENTE = SERVPREST.CDCLIENTE AND DESCSERVPRES.DTSERVPREST = SERVPREST.DTSERVPREST AND DESCSERVPRES.CDEMPRESA = CLIENTE.CDEMPRESA AND DESCSERVPRES.NRSERVPREST = SERVPREST.NRSERVPREST
                LEFT JOIN SERVPRESTDOC ON SERVPREST.CDEMPRESA = SERVPRESTDOC.CDEMPRESA AND SERVPREST.CDFUNC = SERVPRESTDOC.CDFUNC AND SERVPREST.CDCLIENTE = SERVPRESTDOC.CDCLIENTE AND SERVPREST.DTSERVPREST = SERVPRESTDOC.DTSERVPREST AND SERVPREST.NRSERVPREST = SERVPRESTDOC.NRSERVPREST
                LEFT JOIN SERVPRESTDOCASSINA ON SERVPRESTDOC.NRSERVPRESTDOC = SERVPRESTDOCASSINA.NRSERVPRESTDOC
                LEFT JOIN PERFACESSPARC ON PERFACESSPARC.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                LEFT JOIN PERFILACESSO ON PERFACESSPARC.NRPERFILACESSO = PERFILACESSO.NRPERFILACESSO
                GROUP BY 
                FUNCIONARIO.NMFUNC,
                    FUNCIONARIO.CDFUNC ,
                    SERVPREST.DTSERVPREST,
                    SERVPREST.IDSERVPREST,
                    CLIENTE.NMFANTCLIE ,
                    SERVPREST.IDSITSERVPREST,
                    CLIENTE.CDCLIENTE,
                    CLIENTE.IDCLIEATIV,
                    SERVPREST.CDINTECLIE,
                    SERVPREST.NMINTCLIENTE,
                    LICENCAUSO.NMFANTLUSO,
                    SERVPREST.NRPROPOSFAS,
                    SERVPREST.NRVERPROPFAS,
                    PROPOSTAFAS.NRPROPOSFAS || ' - ' || PROPOSTAFAS.DSREFEPROPOSFAS,
                    PROPOSTAFAS.DSREFEPROPOSFAS,
                    PROJETO.CDPROJETO,
                    TIPOSERV.CDTIPOSERV,
                    PROJETO.NMPROJETO,
                    TIPOSERV.NMTIPOSERV, 
                    SERVPREST.DTENTRADA,
                    SERVPREST.HRAUSENCIAALMOCO,
                    SERVPREST.DTSAIDA,
                    SERVPREST.QTHRTOTAL, 
                    SERVPREST.QTHRDESLOCAMENTOFAT,
                    SERVPREST.QTHRATIVIDADEFAT,
                    HORADEC_HORACHAR(HORACHAR_HORADEC(QTHRTOTAL) - HORACHAR_HORADEC(QTHRATIVIDADEFAT)),
                    SERVPRESTDOCASSINA.IDSITASSINATURA,
                    SERVPRESTDOCASSINA.DTASSINATURA,
                    OPERADOR.NRPARCNEGOCIO,
                    SERVPREST.CDEMPRESA, 
                    SERVPREST.NRSERVPREST) REL";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

   
    public function getClienteRelCda()
    {
        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select("DISTINCT CLIENTE.CDCLIENTE, CLIENTE.NMFANTCLIE AS CLIENTECDAREL, 
                    CLIENTE.IDATIVO, CLIENTE.NMFANTCLIE || DECODE(CLIENTE.IDCLIEATIV, 'N', ' - INATIVO') AS DISPLAYCLIENTE")
            ->from('CLIENTE', 'CLIENTE');

        $result = $queryBuilder->execute()->fetchAllAssociative();
        
        return $this->buildResponseData($result);
    }

    private function applyDateFilter($queryBuilder, $dateField, $dateValuesMin, $dateValuesMax): void
    {
        $dateTimeMin = \DateTime::createFromFormat('d/m/Y', $dateValuesMin);

        if ($dateTimeMin !== false && $dateValuesMax === null) {
            $formattedDateMin = $dateTimeMin->format('Y-m-d');
            $queryBuilder->andWhere(
                $queryBuilder->expr()->eq("TRUNC($dateField)", 'TO_DATE(:dateValuesMin, \'YYYY-MM-DD\')')
            )->setParameter('dateValuesMin', $formattedDateMin);
        }

        if ($dateValuesMax !== null) {
            $dateTimeMax = \DateTime::createFromFormat('d/m/Y', $dateValuesMax);

            if ($dateTimeMax !== false) {
                $formattedDateMin = $dateTimeMin->format('Y-m-d');
                $formattedDateMax = $dateTimeMax->format('Y-m-d');

                $queryBuilder->andWhere(
                    $queryBuilder->expr()->andX(
                        $queryBuilder->expr()->gte("TRUNC($dateField)", 'TO_DATE(:dateValuesMin, \'YYYY-MM-DD\')'),
                        $queryBuilder->expr()->lte("TRUNC($dateField)", 'TO_DATE(:dateValuesMax, \'YYYY-MM-DD\')')
                    )
                )->setParameters([
                    'dateValuesMin' => $formattedDateMin,
                    'dateValuesMax' => $formattedDateMax
                ]);
            }
        }
    }

    public function getPropostaCdaRel($parametros)
    {
        // $dateValuesMin = $parametros['DTSERVPREST_MIN'] ?? null;
        // $dateValuesMax = $parametros['DTSERVPREST_MAX'] ?? null;
        $cdClient = isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : null;

        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
        ->select(
            'DISTINCT PROPOSTA.DSREFEPROPOSFAS AS DSREFEPROPOSFASCDAREL',
            'PROPOSTA.NRPROPOSFAS',
            'PROPOSTA.CDCLIENTE AS CLIENTECDAREL',
            'SERVPREST.CDFUNC',
            'SERVPREST.DTSERVPREST'
        )
        ->from('PROPOSTAFAS', 'PROPOSTA')
        ->leftJoin(
            'PROPOSTA',
            'SERVPREST',
            'SERVPREST',
            'SERVPREST.NRVERPROPFAS = PROPOSTA.NRVERPROPFAS AND SERVPREST.NRPROPOSFAS = PROPOSTA.NRPROPOSFAS'
        )
        ->where(
            $queryBuilder->expr()->and(
                $queryBuilder->expr()->isNotNull('PROPOSTA.DSREFEPROPOSFAS'),
                $queryBuilder->expr()->neq('PROPOSTA.DSREFEPROPOSFAS', "'.'")
            )
        );
    
        // $this->applyDateFilter($queryBuilder, 'SERVPREST.DTSERVPREST', $dateValuesMin, $dateValuesMax);
        $this->verifyClientFilter($queryBuilder, 'PROPOSTA.CDCLIENTE', $cdClient);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function verifyClientFilter($queryBuilder, $cdClientField, $cdClient): void
    {
        if (!empty($cdClient)) {
            $queryBuilder->andWhere(
                $queryBuilder->expr()->in($cdClientField, ':CDCLIENTE')
            )->setParameter('CDCLIENTE', $cdClient, Connection::PARAM_STR_ARRAY);
        }
    }

    public function getProjetoCdaRel($parametros)
    {

        $nrProposFas = isset($parametros["NRPROPOSFAS"]) ? $parametros["NRPROPOSFAS"] : null;
        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select('PROJETO.NMPROJETO AS NMPROJETOCDAREL, PROJETO.IDATIVO, PROJETO.NRORG, PROJETO.NRPROPOSFAS, PROJETO.CDCLIENTE, PROJETO.CDPROJETO')
            ->from('PROJETO', 'PROJETO')
            ->leftJoin('PROJETO', 'SERVPREST', 'SERVPREST', 'SERVPREST.NRVERPROPFAS = PROJETO.NRVERPROPFAS AND SERVPREST.NRPROPOSFAS = PROJETO.NRPROPOSFAS')
            ->where($queryBuilder->expr()->eq('PROJETO.IDATIVO', "'S'"))
            ->andWhere($queryBuilder->expr()->in('PROJETO.NRPROPOSFAS', ':NRPROPOSFAS'));

        $queryBuilder->setParameter('NRPROPOSFAS', $nrProposFas, Connection::PARAM_STR_ARRAY);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    public function getTypeOfActivityCdaRel($parametros)
    {
        // $dateValuesMin = $parametros['DTSERVPREST_MIN'] ?? null;
        // $dateValuesMax = $parametros['DTSERVPREST_MAX'] ?? null;
        $cdClient = isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : null;
        $nrProposFas = isset($parametros["NRPROPOSFAS"]) ? $parametros["NRPROPOSFAS"] : null;

        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select('TIPOSERV.NMTIPOSERV AS NMTIPOSERVCDAREL, TIPOSERV.CDTIPOSERV, CONTRATOSERV.CDCLIENTE, CONTRATOSERV.NRPROPOSFAS, CONTRATOSERV.NRVERPROPFAS, 
        CONTRATOSERV.NRSERVCONT, CONTRATOSERV.QTSERVCONT, AGENDASERV.NRORG, AGENDASERV.CDFUNC')
            ->from('TIPOSERV', 'TIPOSERV')
            ->leftJoin('TIPOSERV', 'CONTRATOSERV', 'CONTRATOSERV', 'CONTRATOSERV.CDTIPOSERV = TIPOSERV.CDTIPOSERV')
            ->leftJoin('TIPOSERV', 'AGENDASERV', 'AGENDASERV', 'AGENDASERV.CDTIPOSERV = TIPOSERV.CDTIPOSERV AND AGENDASERV.NRVERPROPFAS = CONTRATOSERV.NRVERPROPFAS
                    AND AGENDASERV.CDTIPOSERV = CONTRATOSERV.CDTIPOSERV AND AGENDASERV.NRSERVCONT = CONTRATOSERV.NRSERVCONT AND AGENDASERV.NRPROPOSFAS = CONTRATOSERV.NRPROPOSFAS')
            ->where(
                $queryBuilder->expr()->and(
                    $queryBuilder->expr()->eq('CONTRATOSERV.IDATIVO', "'S'")
                )
            )
            ->orderBy('TIPOSERV.CDTIPOSERV');

        // $this->applyDateFilterForTypeActivity($queryBuilder, $dateValuesMin, $dateValuesMax);
        $this->verifyClientFilter($queryBuilder, 'CONTRATOSERV.CDCLIENTE', $cdClient);
        $this->verifyProposalFilter($queryBuilder, 'CONTRATOSERV.NRPROPOSFAS', $nrProposFas);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function verifyProposalFilter($queryBuilder, $nrProposFasField, $nrProposFas): void
    {
        if (!empty($nrProposFas)) {
            $queryBuilder->andWhere(
                $queryBuilder->expr()->in($nrProposFasField, ':NRPROPOSFAS')
            )->setParameter('NRPROPOSFAS', $nrProposFas, Connection::PARAM_STR_ARRAY);
        }
    }

    private function applyDateFilterForTypeActivity($queryBuilder, $dateValuesMin, $dateValuesMax): void
    {
        if (!empty($dateValuesMin) && $dateValuesMax === null) {
            $dateTimeMin = \DateTime::createFromFormat('d/m/Y', $dateValuesMin);
            $formattedDateMin = $dateTimeMin->format('Y-m-d');

            $queryBuilder->andWhere(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->gte("AGENDASERV.DTINITRASERV", "TO_DATE(:dateValuesMin, 'YYYY-MM-DD')"),
                    $queryBuilder->expr()->lte("AGENDASERV.DTFINTRASERV", "TO_DATE(:dateValuesMin, 'YYYY-MM-DD')")
                )
            )->setParameter('dateValuesMin', $formattedDateMin);
        } else if (!empty($dateValuesMin) && !empty($dateValuesMax)) {
            $dateTimeMin = \DateTime::createFromFormat('d/m/Y', $dateValuesMin);
            $dateTimeMax = \DateTime::createFromFormat('d/m/Y', $dateValuesMax);
            $formattedDateMin = $dateTimeMin->format('Y-m-d');
            $formattedDateMax = $dateTimeMax->format('Y-m-d');

            $queryBuilder->andWhere(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->gte("AGENDASERV.DTINITRASERV", "TO_DATE(:dateValuesMin, 'YYYY-MM-DD')"),
                    $queryBuilder->expr()->lte("AGENDASERV.DTFINTRASERV", "TO_DATE(:dateValuesMax, 'YYYY-MM-DD')")
                )
            )->setParameters([
                'dateValuesMin' => $formattedDateMin,
                'dateValuesMax' => $formattedDateMax
            ]);
        }
    }

    public function getFuncRelCda($parametros)
    {

        $nrperfilacesso = isset($parametros["NRPERFILACESSO"]) ? $parametros["NRPERFILACESSO"] : null;
        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select('DISTINCT FUNCIONARIO.CDFUNC AS CDOPERADORCDAREL, FUNCIONARIO.NMFUNC AS NMOPERADORCDAREL, PERFILACESSO.NMPERFILACESSO, PERFILACESSO.NRPERFILACESSO' )
            ->from('FUNCIONARIO', 'FUNCIONARIO')
            ->leftJoin('FUNCIONARIO', 'OPERADOR', 'OPERADOR', 'OPERADOR.CDFUNC = FUNCIONARIO.CDFUNC')
            ->leftJoin('OPERADOR', 'PARCNEGOCIO', 'PARCNEGOCIO', 'OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
            ->leftJoin('PARCNEGOCIO', 'PERFACESSPARC', 'PERFACESSPARC', 'PERFACESSPARC.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
            ->leftJoin('PERFACESSPARC', 'PERFILACESSO', 'PERFILACESSO', 'PERFACESSPARC.NRPERFILACESSO = PERFILACESSO.NRPERFILACESSO')
            ->where($queryBuilder->expr()->eq('FUNCIONARIO.IDATIVO', "'S'"));

        $queryBuilder->setParameter('NRPERFILACESSO', $nrperfilacesso);
        $this->verifyPerfilFilter($queryBuilder, 'PERFILACESSO.NRPERFILACESSO', $nrperfilacesso);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function verifyPerfilFilter($queryBuilder, $nrperfilacessoField, $nrperfilacesso): void
    {
        if (!empty($nrperfilacesso)) {
            $queryBuilder->andWhere(
                $queryBuilder->expr()->in($nrperfilacessoField, ':NRPERFILACESSO')
            )->setParameter('NRPERFILACESSO', $nrperfilacesso, Connection::PARAM_STR_ARRAY);
        }
    }

    public function getPerfAcesRelCda()
    {

        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
        ->select('NRPERFILACESSO, NMPERFILACESSO')
        ->from('PERFILACESSO', 'PERF')
        ->where($queryBuilder->expr()->eq('PERF.IDATIVO', "'S'"))
        ->orderBy('NMPERFILACESSO', 'ASC');

        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }
    
}
