<?php
namespace SM\Service;

use SM\Factory\DashboardFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DataSource\DataSet;

class DashboardService
{

    protected $DashboardFactory;
    protected $Environment;
    protected $newCode;

    public function __construct(DashboardFactory $DashboardFactory, Environment $Environment, NewCode $newCode)
    {
        $this->DashboardFactory  = $DashboardFactory;
        $this->Environment       = $Environment;
        $this->newCode           = $newCode;
    }

    public function getApprovedContracts($parametros)
    {
        return $this->DashboardFactory->getApprovedContracts($parametros);
    }

    public function getTerminatedContracts($parametros)
    {
        return $this->DashboardFactory->getTerminatedContracts($parametros);
    }

    public function getRenegotiatedContracts($parametros)
    {
        return $this->DashboardFactory->getRenegotiatedContracts($parametros);
    }

    public function getContractReminder()
    {
        return $this->DashboardFactory->getContractReminder();
    }
    
    public function getContractsCloseToExpiration()
    {
        return $this->DashboardFactory->getContractsCloseToExpiration();
    }

    public function getContractsPendingBilling($parametros)
    {
        $params = array(
            "DTINI" => ($parametros['DTINI'] == 1) ? null : $parametros['DTINI'],
            "DTFIN" => ($parametros['DTFIN'] == 1) ? null : $parametros['DTFIN']
        );

        return $this->DashboardFactory->getContractsPendingBilling($params);
    }

    public function getChartBusinessPartner($parametros)
    {
        return $this->DashboardFactory->getChartBusinessPartner($parametros);
    }

    public function getChartBillingScope($parametros)
    {

        return $this->DashboardFactory->getChartBillingScope($parametros);
    }

    public function getChartContractItems($parametros)
    {
        return $this->DashboardFactory->getChartContractItems($parametros);
    }

    public function getMonthlyHistory()
    {
        return $this->DashboardFactory->getMonthlyHistory();
    }

    public function getBusinessPartnerDetailed($parametros)
    {
        $params = array(
            "DTINICIO"      =>  $parametros['DTINICIO'],
            "DTFINAL"       =>  $parametros['DTFINAL'],
            "NRPARCNEGOCIO" =>  $parametros['NRPARCNEGOCIO']
        );
        return $this->DashboardFactory->getBusinessPartnerDetailed($params);
    }

    public function getPartners()
    {
        return $this->DashboardFactory->getPartners();
    }

    public function getScopes()
    {
        return $this->DashboardFactory->getScopes();
    }

    public function getBillingScopeDetailed($parametros)
    {

        $params = array(
            "DTINICIO"      =>  $parametros['DTINICIO'],
            "DTFINAL"       =>  $parametros['DTFINAL'],
            "NRTIPOORCAMENTO" =>  $parametros['NRTIPOORCAMENTO']
        );
        return $this->DashboardFactory->getBillingScopeDetailed($params);
    }

    public function amountOfProposalAccepted($parametros)
    {
        return $this->DashboardFactory->amountOfProposalAccepted($parametros);
    }

    public function valueOfAcceptedProposal($parametros)
    {
        return $this->DashboardFactory->valueOfAcceptedProposal($parametros);
    }

    public function valueOfAcceptedProposalXBusinessPartner($parametros)
    {
        return $this->DashboardFactory->valueOfAcceptedProposalXBusinessPartner($parametros);
    }

    public function getContractItemsDetailed($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros['DTINICIO'],
            "DTFINAL"    =>  $parametros['DTFINAL'],
            "CDSERVORCA" =>  $parametros['CDSERVORCA']
        );
        return $this->DashboardFactory->getContractItemsDetailed($params);
    }

    public function getContractItems()
    {
        return $this->DashboardFactory->getContractItems();
    }

    // Métodos usados no Novo Dashboard com Gráficos utilizando o componente ECharts

    public function getQtdPropostasAceitasPorEscopo($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getQtdPropostasAceitasPorEscopo($params);
    }

    public function getValorPropostasAceitasPorEscopo($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getValorPropostasAceitasPorEscopo($params);
    } 

    public function getValorPropostasAceitasPorParceiro($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getValorPropostasAceitasPorParceiro($params);
    }

    public function getContratosAprovadosPorEscopo($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getContratosAprovadosPorEscopo($params);
    }

    public function getContratosRescindidosPorEscopo($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getContratosRescindidosPorEscopo($params);
    }

    public function getContratosRenegociadosPorEscopo($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getContratosRenegociadosPorEscopo($params);
    }

    public function getContratroFaturamentoPendente($parametros)
    {
        $params = array(
            "DTINICIO"   =>  '',
            "DTFINAL"    =>  '',
        );
        return $this->DashboardFactory->getContratroFaturamentoPendente($params);
    }

    public function getFaturamentosValorFatPorParceiro($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getFaturamentosValorFatPorParceiro($params);
    } 

    public function getFaturamentosValorFatPorEscopo($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getFaturamentosValorFatPorEscopo($params);
    } 

    public function getFaturamentosValorFatPorItemContrato($parametros)
    {
        $params = array(
            "DTINICIO"   =>  $parametros["DTRANGEINICIO"],
            "DTFINAL"    =>  $parametros["DTRANGEFINAL"],
        );
        return $this->DashboardFactory->getFaturamentosValorFatPorItemContrato($params);
    }

    public function getFaturamentosValorFaturadoPorMes()
    {
        return $this->DashboardFactory->getFaturamentosValorFaturadoPorMes();
    }
}
