<?php

namespace SM\Service;

use SM\Factory\Sm00029000LocalAtendimentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use SM\Service\FileServerService;
use SM\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Sm00029000LocalAtendimentoService
{
    protected $localAtendimentoFactory;
    protected $Environment;
    protected $newCode;
    protected $fileServerService;

    public function __construct(EntityManager $entityManager, Sm00029000LocalAtendimentoFactory $localAtendimentoFactory, Environment $Environment, NewCode $newCode, FileServerService $fileServerService, $fileServerUrl)
    {
        $this->localAtendimentoFactory = $localAtendimentoFactory;
        $this->connection             = $entityManager->getConnection();
        $this->Environment            = $Environment;
        $this->newCode                = $newCode;
        $this->fileServerService      = $fileServerService;
        $this->fileServerUrl          = $fileServerUrl;
    }

    public function getTipLocAtendEstruturam()
    {
        return $this->localAtendimentoFactory->getTipLocAtendEstruturam();
    }

    public function getTipoLocalAtendimento()
    {
        return $this->localAtendimentoFactory->getTipoLocalAtendimento();
    }

    public function findLocalAtendimentoCentroCusto()
    {
        return $this->localAtendimentoFactory->findLocalAtendimentoCentroCusto();
    }

    public function findPais()
    {
        return $this->localAtendimentoFactory->findPais();
    }
  
    public function findEstado($parametro)
    {
        return $this->localAtendimentoFactory->findEstado($parametro);
    }

    public function findMunicipio($parametro)
    {
        return $this->localAtendimentoFactory->findMunicipio($parametro);
    }

    /**
     * Deleta a Hierarquia da Estrutura.
     */
    public function deletarHierarquiaOrg($nrestruturam) {
        $nrorg = $this->Environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam
        );
        
        $this->localAtendimentoFactory->deletarHierarquiaOrg($params);
    }

    /**
     * Deleta a Estrutura H.
     */
    public function deletarEstruturaH($nrestruturam) {
        $nrorg = $this->Environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam
        );
        
        $this->localAtendimentoFactory->deletarEstruturaH($params);
    }

    /**
     * Deleta a Estrutura M.
     */
    public function deletarEstruturaM($nrestruturam) {
        $nrorg = $this->Environment->getNrOrg();
        
        $params = array(
            "NRESTRUTURAM"    => $nrestruturam
        );
        
        $this->localAtendimentoFactory->deletarEstruturaM($params);
    }   

    /**
     * Deleta o endereço de Hierarquia da Estrutura.
     */
    public function deletarEnderecoParc($nrparncnegocio) {
        $nrorg = $this->Environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio
        );
        
        $this->localAtendimentoFactory->deletarEnderecoParc($params);
    }

    /**
     * Deleta o Parceiro de Negócio.
     */
    public function deletarParcNegocio($nrparncnegocio) {
        $nrorg = $this->Environment->getNrOrg();
        
        $params = array(
            "NRPARCNEGOCIO"       => $nrparncnegocio
        );
        
        $this->localAtendimentoFactory->deletarParcNegocio($params);
    }

    public function deleteLocalAtendimento($rows) {
        
        $this->connection->beginTransaction();

        try {
            foreach($rows as $estrutura){

                
                $nrestruturam = $estrutura['NRESTRUTURAM'];
                
                $params = array(
                    "NRESTRUTURAM" => $nrestruturam,
                );
            
                $filhos = $this->localAtendimentoFactory->recebefilhos($params);
            
                
                foreach($filhos as $filho){
                    
                    $estruturam = $filho['NRESTRUTURAM'];
                    $parcnegocio = $filho['NRPARCNEGOCIO'];
                    
                    /* Hierarquia */
                    $this->deletarHierarquiaOrg($estruturam);
                    
                    /* Estrutura H */
                    $this->deletarEstruturaH($estruturam); 
                    
                    /* Estrutura M */
                    $this->deletarEstruturaM($estruturam);
                    
                    /* Endereço do Parceiro de Negócio */
                    $this->deletarEnderecoParc($parcnegocio);
                    
                    /* Parceiro de Negócio */ 
                    $this->deletarParcNegocio($parcnegocio);
                }
            }

            // var_dump($estruturam);
            // var_dump($parcnegocio); die;

            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getParcNegocioCliente()
    {
        return $this->localAtendimentoFactory->getParcNegocioCliente();
    }

    public function getLocalAtendimentoFilial()
    {
        return $this->localAtendimentoFactory->getLocalAtendimentoFilial();
    }

    public function getLocalAtendimentoEmpresa()
    {
        return $this->localAtendimentoFactory->getLocalAtendimentoEmpresa();
    }

    public function getLocalAtendimentoInscrEstad()
    {
        return $this->localAtendimentoFactory->getLocalAtendimentoInscrEstad();
    }

    public function getLocalAtendimentoInscrMunicip()
    {
        return $this->localAtendimentoFactory->getLocalAtendimentoInscrMunicip();
    }

    // public function montarFiltro($find)
    // {
    //     $operador = $this->Environment->getCdOperador();
    //     if (empty($find)) {
    //         return '1 = 1';
    //     }
    //     $whereClause = [];
    //     foreach ($find as $column => $condition) {
    //         $operator = key($condition);
    //         $value = $condition[$operator];

    //         if ($value === 'OPERADOR_LOGADO') {
    //             $value = $operador;
    //         }

    //         if (in_array($operator, ["IN", "NOT_IN"])) {
    //             $value = "($value)";
    //         } else {
    //             $value = "'$value'";
    //         }

    //         $whereClause[] = "$column $operator $value";
    //     }

    //     return implode(" AND ", $whereClause);
    // }

    // public function getBairro($filtro, $parceiro)
    // {
    //     $where = $this->montarFiltro($filtro);
    //     $parametros = array(
    //         "NRPARCNEGOCIO" => isset($parceiro['NRPARCNEGOCIO']) ? $parceiro['NRPARCNEGOCIO'] : null
    //     );
    //     // var_dump($where, $parceiro); die;
    //     return $this->localAtendimentoFactory->getBairro($where, $parametros);
    // }


    public function montarFiltro($find)
    {
        $operador = $this->Environment->getCdOperador();
        if (empty($find)) {
            return '1 = 1';
        }
        
        $whereClause = [];
        
        foreach ($find as $column => $condition) {
            if (!is_array($condition) || empty($condition)) {
                continue;
            }

            $operator = key($condition);
            $value = $condition[$operator];

            if ($value === 'OPERADOR_LOGADO') {
                $value = $operador;
            }

            if (in_array($operator, ["IN", "NOT IN"])) {
                if (is_array($value)) {
                    $value = "('" . implode("','", array_map('addslashes', $value)) . "')";
                } else {
                    $value = "('$value')";
                }
            } else {
                $value = "'" . addslashes($value) . "'";
            }

            $whereClause[] = "$column $operator $value";
        }

        return implode(" AND ", $whereClause);
    }

    // public function getBairro($filtro, $parceiro)
    // {
    //     $where = $this->montarFiltro($filtro);
    //     $parametros = [
    //         "NRPARCNEGOCIO" => $parceiro['NRPARCNEGOCIO'] ?? null
    //     ];

    //     return $this->localAtendimentoFactory->getBairro($where, $parametros);
    // }

    public function getBairro($filtro, $parceiro, $params)
    {
        $where = $this->montarFiltro($filtro);

        // $parc = [
        //     "NRPARCNEGOCIO" => $parceiro['NRPARCNEGOCIO'] ?? null
        // ];

        return $this->localAtendimentoFactory->getBairro($where, $params);
    }

    public function findCodeDistrict($parametros)
    {
        return $this->localAtendimentoFactory->findCodeDistrict($parametros);
    }

    public function findBairro($parametros)
    {
        return $this->localAtendimentoFactory->findBairro($parametros);
    }
}
